import { makeAbstractionLoader, UnknownNodeTypeError } from '../build/helpers.js';
import { defaultSettingsForBuild as defaultSettingsForBuild$1 } from '../build/build.js';
import '../../../WebPd_compiler/dist/src/stdlib/fs.js';
import '../../../WebPd_compiler/dist/src/compile/precompile/nodes.js';
import '../nodes/nodes/osc~-phasor~.js';
import '../nodes/nodes/clip~.js';
import '../nodes/nodes/sig~.js';
import '../nodes/nodes/samphold~.js';
import '../nodes/nodes/snapshot~.js';
import '../nodes/nodes/vline~.js';
import '../nodes/nodes/line~.js';
import '../nodes/nodes/line.js';
import '../nodes/nodes/tabread.js';
import '../nodes/nodes/tabwrite.js';
import '../nodes/nodes/tabread~-tabread4~.js';
import '../nodes/nodes/tabplay~.js';
import '../nodes/nodes/readsf~.js';
import '../nodes/nodes/writesf~.js';
import '../nodes/nodes/filters-bp~.js';
import '../nodes/nodes/throw~-catch~-send~-receive~.js';
import '../nodes/nodes/metro.js';
import '../nodes/nodes/timer.js';
import '../nodes/nodes/delay.js';
import '../nodes/nodes/controls-float.js';
import '../nodes/nodes/controls-bang.js';
import '../nodes/nodes/controls-atoms.js';
import '../nodes/nodes/float-int.js';
import '../nodes/nodes/delread~-delread4~.js';
import '../nodes/nodes/delwrite~.js';
import '../nodes/nodes/filters-real~.js';
import '../nodes/nodes/filters-complex~.js';
import '../nodes/nodes/filters-hip~.js';
import '../nodes/nodes/filters-lop~.js';
import '../nodes/nodes/filters-vcf~.js';
import '../nodes/nodes/msg.js';
import '../nodes/nodes/list.js';
import '../nodes/nodes/symbol.js';
import '../nodes/nodes/send-receive.js';
import '../nodes/nodes/soundfiler.js';
import '../nodes/nodes/change.js';
import '../nodes/nodes/moses.js';
import '../nodes/nodes/clip.js';
import '../nodes/nodes/route.js';
import '../nodes/nodes/spigot.js';
import '../nodes/nodes/until.js';
import '../nodes/nodes/random.js';
import '../nodes/nodes/pipe.js';
import '../nodes/nodes/pack.js';
import '../nodes/nodes/expr-expr~.js';
import '../nodes/nodes/binop.js';
import '../compile-dsp-graph/to-dsp-graph.js';
import { urlDirName } from './url-helpers.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
const defaultSettingsForBuild = (patchUrl) => ({
    ...defaultSettingsForBuild$1(),
    abstractionLoader: makeUrlAbstractionLoader(patchUrl),
});
/**
 * Helper to build an abstraction loader from a patch url.
 * The returned loader will :
 * - use the root url of the patch to resolve relative paths for abstractions.
 * - suffix all abstraction names with .pd if they don't already have an extension.
 *
 * @param patchUrl
 * @returns
 */
const makeUrlAbstractionLoader = (patchUrl) => {
    const rootUrl = urlDirName(patchUrl);
    return makeAbstractionLoader(async (nodeType) => {
        const url = `${rootUrl}/${(nodeType.endsWith('.pd') ? nodeType : `${nodeType}.pd`)}`;
        const response = await fetch(url);
        if (!response.ok) {
            throw new UnknownNodeTypeError(nodeType);
        }
        return await response.text();
    });
};

export { defaultSettingsForBuild, makeUrlAbstractionLoader };
