import { ast, ConstVar, AnonFunc, Var, Sequence, Func } from '../../../../WebPd_compiler/dist/src/ast/declare.js';
import { commonsWaitEngineConfigure, commonsWaitFrame } from '../../../../WebPd_compiler/dist/src/stdlib/commons.js';
import '../../../../WebPd_compiler/dist/src/stdlib/fs.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/nodes.js';
import { assertOptionalString } from '../validation.js';
import { EMPTY_BUS_NAME, build, controlsCoreVariableNames, controlsCore } from './controls-base.js';
import { messageBuses } from '../global-code/buses.js';
import { bangUtils } from '../global-code/core.js';
import { generateVariableNamesNodeType } from '../variable-names.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: ({ args: [init, receive, send] }) => ({
        outputOnLoad: !!init,
        sendBusName: assertOptionalString(send) || EMPTY_BUS_NAME,
        receiveBusName: assertOptionalString(receive) || EMPTY_BUS_NAME,
    }),
    build,
};
// ------------------------------- node implementation ------------------------------ //
const variableNames = generateVariableNamesNodeType('bang', ['receiveMessage']);
const nodeImplementation = {
    initialization: ({ snds, state, node: { args }, }) => ast `
        ${ConstVar(controlsCoreVariableNames.stateClass, state, `{
            value: msg_create([]),
            receiveBusName: "${args.receiveBusName}",
            sendBusName: "${args.sendBusName}",
            messageReceiver: ${controlsCoreVariableNames.defaultMessageHandler},
            messageSender: ${controlsCoreVariableNames.defaultMessageHandler},
        }`)}
    
        commons_waitEngineConfigure(() => {
            ${state}.messageReceiver = ${AnonFunc([Var('Message', 'm')]) `
                ${variableNames.receiveMessage}(${state}, m)
            `}
            ${state}.messageSender = ${snds.$0}
            ${controlsCoreVariableNames.setReceiveBusName}(${state}, "${args.receiveBusName}")
        })

        ${args.outputOnLoad ?
        `commons_waitFrame(0, () => ${snds.$0}(msg_bang()))` : null}
    `,
    messageReceivers: ({ state }) => ({
        '0': AnonFunc([Var('Message', 'm')]) `
            ${variableNames.receiveMessage}(${state}, m)
            return
        `,
    }),
    dependencies: [
        bangUtils,
        messageBuses,
        commonsWaitEngineConfigure,
        commonsWaitFrame,
        controlsCore,
        () => Sequence([
            Func(variableNames.receiveMessage, [
                Var(controlsCoreVariableNames.stateClass, 'state'),
                Var('Message', 'm'),
            ], 'void') `
                if (${controlsCoreVariableNames.setSendReceiveFromMessage}(state, m) === true) {
                    return
                }
                
                ${ConstVar('Message', 'outMessage', 'msg_bang()')}
                state.messageSender(outMessage)
                if (state.sendBusName !== "${EMPTY_BUS_NAME}") {
                    msgBusPublish(state.sendBusName, outMessage)
                }
                return
            `
        ]),
    ],
};

export { builder, nodeImplementation };
