import { ast, ConstVar, Sequence, Class, Var, Func } from '../../../../WebPd_compiler/dist/src/ast/declare.js';
import { commonsWaitEngineConfigure } from '../../../../WebPd_compiler/dist/src/stdlib/commons.js';
import '../../../../WebPd_compiler/dist/src/stdlib/fs.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/nodes.js';
import { bufWriteRead } from '../../../../WebPd_compiler/dist/src/stdlib/buf.js';
import { assertOptionalString, assertOptionalNumber } from '../validation.js';
import { delayBuffers } from '../global-code/delay-buffers.js';
import { computeUnitInSamples } from '../global-code/timing.js';
import { generateVariableNamesNodeType } from '../variable-names.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
// TODO : Implement 4-point interpolation for delread4
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: ({ args }) => ({
        delayName: assertOptionalString(args[0]) || '',
        initDelayMsec: assertOptionalNumber(args[1]) || 0,
    }),
    build: () => ({
        inlets: {
            '0': { type: 'signal', id: '0' },
        },
        outlets: {
            '0': { type: 'signal', id: '0' },
        },
    }),
    configureMessageToSignalConnection: (inletId, { initDelayMsec }) => {
        if (inletId === '0') {
            return { initialSignalValue: initDelayMsec };
        }
        return undefined;
    },
};
// ------------------------------- node implementation ------------------------------ //
const makeNodeImplementation = () => {
    const variableNames = generateVariableNamesNodeType('delread', ['setDelayName']);
    return {
        initialization: ({ node: { args }, state }) => ast `
            ${ConstVar(variableNames.stateClass, state, `{
                delayName: '',
                buffer: DELAY_BUFFERS_NULL,
            }`)}

            commons_waitEngineConfigure(() => {
                if ("${args.delayName}".length) {
                    ${variableNames.setDelayName}(${state}, "${args.delayName}", () => {
                        ${state}.buffer = DELAY_BUFFERS.get(${state}.delayName)
                    })
                }
            })
        `,
        inlineLoop: ({ globs, ins, state }) => ast `buf_readSample(${state}.buffer, toInt(Math.round(
            Math.min(
                Math.max(computeUnitInSamples(${globs.sampleRate}, ${ins.$0}, "msec"), 0), 
                toFloat(${state}.buffer.length - 1)
            )
        )))`,
        dependencies: [
            computeUnitInSamples,
            delayBuffers,
            commonsWaitEngineConfigure,
            bufWriteRead,
            () => Sequence([
                Class(variableNames.stateClass, [
                    Var('string', 'delayName'),
                    Var('buf_SoundBuffer', 'buffer'),
                ]),
                Func(variableNames.setDelayName, [
                    Var(variableNames.stateClass, 'state'),
                    Var('string', 'delayName'),
                    Var('SkedCallback', 'callback'),
                ], 'void') `
                    if (state.delayName.length) {
                        state.buffer = DELAY_BUFFERS_NULL
                    }
                    state.delayName = delayName
                    if (state.delayName.length) {
                        DELAY_BUFFERS_get(state.delayName, callback)
                    }
                `
            ]),
        ],
    };
};
const builders = {
    'delread~': builder,
    'delread4~': builder,
};
const nodeImplementations = {
    'delread~': makeNodeImplementation(),
    'delread4~': makeNodeImplementation(),
};

export { builders, nodeImplementations };
