import { Sequence, Class, Var, Func, ast, ConstVar, AnonFunc } from '../../../../WebPd_compiler/dist/src/ast/declare.js';
import '../../../../WebPd_compiler/dist/src/stdlib/fs.js';
import '../../../../WebPd_compiler/dist/src/compile/precompile/nodes.js';
import { assertOptionalNumber } from '../validation.js';
import { bangUtils } from '../global-code/core.js';
import { roundFloatAsPdInt } from '../global-code/numbers.js';
import { coldFloatInletWithSetter } from '../standard-message-receivers.js';
import { generateVariableNamesNodeType } from '../variable-names.js';

/*
 * Copyright (c) 2022-2023 Sébastien Piquemal <sebpiq@protonmail.com>, Chris McCormick.
 *
 * This file is part of WebPd
 * (see https://github.com/sebpiq/WebPd).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
// TODO: proper support for $ args
// TODO: simple number - shortcut for float
// ------------------------------- node builder ------------------------------ //
const builder = {
    translateArgs: (pdNode) => ({
        value: assertOptionalNumber(pdNode.args[0]) || 0,
    }),
    build: () => ({
        inlets: {
            '0': { type: 'message', id: '0' },
            '1': { type: 'message', id: '1' },
        },
        outlets: {
            '0': { type: 'message', id: '0' },
        },
        isPushingMessages: true,
    }),
};
// ------------------------------- generateDeclarations ------------------------------ //
const variableNames = generateVariableNamesNodeType('float_int', [
    'setValueInt',
    'setValueFloat'
]);
const nodeCore = () => Sequence([
    Class(variableNames.stateClass, [
        Var('Float', 'value')
    ]),
    Func(variableNames.setValueInt, [
        Var(variableNames.stateClass, 'state'),
        Var('Float', 'value'),
    ], 'void') `
        state.value = roundFloatAsPdInt(value)
    `,
    Func(variableNames.setValueFloat, [
        Var(variableNames.stateClass, 'state'),
        Var('Float', 'value'),
    ], 'void') `
        state.value = value
    `
]);
const makeNodeImplementation = (setValueVariableName) => {
    const initialization = ({ node: { args }, state }) => ast `
            ${ConstVar(variableNames.stateClass, state, `{
                value: 0,
            }`)}
            ${setValueVariableName}(${state}, ${args.value})
        `;
    // ------------------------------- messageReceivers ------------------------------ //
    const messageReceivers = ({ snds, state, }) => ({
        '0': AnonFunc([Var('Message', 'm')]) `
            if (msg_isMatching(m, [MSG_FLOAT_TOKEN])) {
                ${setValueVariableName}(${state}, msg_readFloatToken(m, 0))
                ${snds.$0}(msg_floats([${state}.value]))
                return 

            } else if (msg_isBang(m)) {
                ${snds.$0}(msg_floats([${state}.value]))
                return
                
            }
        `,
        '1': coldFloatInletWithSetter(setValueVariableName, state),
    });
    return {
        initialization: initialization,
        messageReceivers: messageReceivers,
    };
};
// ------------------------------------------------------------------- //
const builders = {
    float: builder,
    f: { aliasTo: 'float' },
    int: builder,
    i: { aliasTo: 'int' },
};
const nodeImplementations = {
    float: {
        ...makeNodeImplementation(variableNames.setValueFloat),
        dependencies: [bangUtils, nodeCore],
    },
    int: {
        ...makeNodeImplementation(variableNames.setValueInt),
        dependencies: [roundFloatAsPdInt, bangUtils, nodeCore],
    },
};

export { builders, nodeImplementations };
